
#ifndef Pythia8_SplittingLibrary_H
#define Pythia8_SplittingLibrary_H

#define DIRE_SPLITTINGLIBRARY_VERSION "0.900"

#include "Pythia8/Basics.h"
#include "Pythia8/Pythia.h"
#include "Pythia8/BeamParticle.h"
#include "Pythia8/ParticleData.h"
#include "Pythia8/PythiaStdlib.h"
#include "Pythia8/Settings.h"
#include "Pythia8/StandardModel.h"
#include "Splittings.h"
//#include "MGSplittings.h"

namespace Pythia8 {

class DireSpace;
class DireTimes;
 
//==========================================================================

class SplittingLibrary {

public:  

  // Constructor and destructor.
  SplittingLibrary() : settingsPtr(0), particleDataPtr(0), rndmPtr(0),
    beamAPtr(0), beamBPtr(0) {}
 ~SplittingLibrary() { clear();}

  // Initialisation.
  void init(Settings* settings, ParticleData* particleData, Rndm* rndm,
    BeamParticle* beamA, BeamParticle* beamB);
  void initFSR();
  void initISR();
  void clear();

  void setTimesPtr (DireTimes* fsrIn) {
    for ( int i = 0; i < int(splittings.size()); ++i)
      splittings[i]->setTimesPtr (fsrIn);
  }
  void setSpacePtr (DireSpace* isrIn) {
    for ( int i = 0; i < int(splittings.size()); ++i)
      splittings[i]->setSpacePtr (isrIn);
  }

  vector< Splitting* > getSplittingNames() { return splittings;}

  // Overload index operator to access element of splitting vector.
  Splitting* operator[](string id);
  const Splitting* operator[](string id) const;

  // Generate name for a splitting
  int getSplittingRadBefID(const Event& event, int rad, int emt);

  // Generate name for a splitting
  string getSplittingName(const Event& event, int rad, int emt);

private:

  vector< Splitting* > splittings;
  Settings* settingsPtr;
  ParticleData* particleDataPtr;
  Rndm* rndmPtr;
  BeamParticle* beamAPtr;
  BeamParticle* beamBPtr;

};

} // end namespace Pythia8

#endif
