#!/bin/bash

###############################################################################
# FUNCTION TO LIST A FILE
###############################################################################

list_file () {

echo ""
echo "-------------------------------------------------------------------------"
echo "  List file $1                                                      "
echo ""

cat $1

echo ""
echo "-------------------------------------------------------------------------"

}

###############################################################################
# FUNCTION TO UPDATE THE INPUT CARD
###############################################################################

update_input_card () {

echo "-------------------------------------------------------------------------"
echo "  Update input card setting to $1  =  $2                                 "

local setting="$1"
local value="$2"
local card="$3"
local stamp="$(echo $RANDOM % 1000 + 1 | bc)"
local tmp_card="params-$stamp.params"
while [ -e $tmp_card ] ; do stamp="$(echo $RANDOM % 1000 + 1 | bc)" ; tmp_card="params-$stamp.params" ; done ;

grep -v -- "$setting " $card > $tmp_card
echo "$setting  =  $value" >> $tmp_card

mv $tmp_card $card

echo "-------------------------------------------------------------------------"

}

###############################################################################
# FUNCTION TO START MULTIPLE RIVET RUNS
###############################################################################

run_rivet () {
  until [ -z "$1" ]
  do
    in="$(echo $1 | sed 's|?| |g')"
    $in > /dev/null 2>&1 &
#    $in &
    shift
  done
}


###############################################################################
# FUNCTION TO RUN GENERATOR AND PRODUCE HISTOGRAM FILES
###############################################################################

run_generation () {

echo ""
echo "-------------------------------------------------------------------------"
echo "RUN EVENT GENERATION AND ANALYSIS"

# Input settings file
local params="$1"
# Generator terminal output
local logfile="$2"
touch $logfile
if [ -e $logfile ] ; then rm $logfile ; fi ;
# Rivet histogram output
local file="$3"

# Setup Rivet command.
local stamp="$(echo $RANDOM % 1000 + 1 | bc)"
local hepmcfile="run_genration-$stamp.fifo"
while [ -e $hepmcfile ] ; do stamp="$(echo $RANDOM % 1000 + 1 | bc)" ; hepmcfile="run_generation-$stamp.fifo" ; done ;
mkfifo $hepmcfile
local rivetExecString="python?$RIVETBIN/rivet?-v?--analysis-path-append=$RIVETINST?-a?$RIVET_ANALYSIS_NAME?-H?$file?$hepmcfile "

# Setup generator command.
local generatorExecString="$PYTHIAEXEC/dire01 $params $hepmcfile"
#local generatorExecString="$PYTHIAEXEC/dire01 $params test.hepmc"

# Source Rivet.
echo "  Source rivet environment variables"
RIVET_ENV="$(find $RIVETINST -name 'rivetenv.sh' | tail -n 1)"
echo "  source $RIVET_ENV"
source $RIVET_ENV

echo "  Pythia8 terminal output written to $logfile"
echo "  Running Pythia8 with command:      $generatorExecString"
echo "  Running RIVET with command:        $rivetExecString"

# Begin generation.
list_file $params > $logfile
$generatorExecString | tee -a $logfile 2>&1 &
run_rivet $rivetExecString
#$generatorExecString &

# wait until both runs are finished
local tmp_jobs="jobs-$stamp.log"
while true ; do jobs -l > $tmp_jobs
  # at least one job finished with error exit code:
  if [[ "$(cat $tmp_jobs | grep "Exit" | wc -l)" != "0" ]] ; then
    echo "ERROR: fail to run Pythia8 or Rivet" | tee -a $logfile
    rm $tmp_jobs
    rm $hepmcfiles
    rm $file*.yoda
    exit 1
  fi
  if [[ "$(cat $tmp_jobs | wc -l)" == "0" ]] ; then
    echo "INFO: Pythia8 and Rivet finished successfully with commands $generatorExecString $rivetExecString" | tee -a $logfile
    break ;
  fi;
  sleep 3
done

rm $tmp_jobs
rm $hepmcfile

echo ""
echo "-------------------------------------------------------------------------"

}

run () {

echo ""
echo "========================================================================="
echo "               THIS IS A RUN SCRIPT FOR PYTHIA+Rivet                     "
echo ""

  pyd="$1"
  local proc="$2"
  local lhef1="$3"
  local seed="$4"
  local mode="$proc"

  # Set paths
  PYTHIAEXEC="$pyd"

  cd $PYTHIAEXEC

  RIVETINST=/nfs/farm/g/theory/qcdsim/sp/hepsoft/RIVET2.gcc-4.8.0
  RIVETBIN=/nfs/farm/g/theory/qcdsim/sp/hepsoft/RIVET2.gcc-4.8.0/bin
  source $RIVETINST/rivetenv.sh
  export RIVET_REF_PATH=$RIVETINST/lib64
  export RIVET_ANALYSIS_PATH=$RIVETINST/lib64

  # prepare inputs
  local params="run.params"
  cp ${mode}.cmnd $params
  nevents="10000"

  update_input_card "Main:numberOfEvents" "$nevents" $params
  update_input_card "Random:seed" "$seed" $params
  update_input_card "Beams:LHEF" "$lhef1" $params;

  RIVET_ANALYSIS_NAME="MC_XS"
  if [[ "$mode" == "dis" ]] ; then
    RIVET_ANALYSIS_NAME="MC_XS?-a?MC_QCD_PARTONS"
  elif [[ "$mode" == "lep" ]] ; then
    RIVET_ANALYSIS_NAME="MC_XS?-a?MC_QCD_PARTONS"
  elif [[ "$mode" == "lhc" ]] ; then
    RIVET_ANALYSIS_NAME="MC_XS?-a?MC_QCD_PARTONS?-a?MC_ZINC?-a?MC_HINC?-a?MC_WINC"
  elif [[ "$mode" == "jets" ]] ; then
    RIVET_ANALYSIS_NAME="MC_XS?-a?MC_QCD_PARTONS"
  fi

  # compile
  rm dire01
  make dire01

  # prepare outputs
  log="dire.log"
  yoda="${proc}_${mode}_direhists.yoda"

  cat $params

  # run
  time run_generation $params $log $yoda

  # clean up, done
  rm $params

echo ""
echo "========================================================================="
echo ""

}

run "$@"
